<?php
/**
 * Troll_Driver defines an API for implementing storage backends for
 * Troll.
 *
 * Copyright 2003-2005 Todd Merritt <tmerritt@email.arizona.edu>
 * Copyright 2003-2005 Chris Bowlby <excalibur@hub.org>
 * Copyright 2003-2005 Max Kalika <max@gentoo.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: troll/lib/Driver.php,v 1.7 2005/01/03 14:35:35 jan Exp $
 *
 * @author  Todd Merritt <tmerritt@email.arizona.edu>
 * @author  Chris Bowlby <excalibur@hub.org>
 * @author  Max Kalika <max@gentoo.org>
 * @version $Revision: 1.7 $
 * @since   Troll 0.1
 * @package Troll
 */
class Troll_Driver {

    /**
     * Hash containing connection parameters for this backend.
     *
     * @var array $_params
     */
    var $_params = array();

    /**
     * The user name to whom the Troll subscriptions belong.
     *
     * @var string $_user
     */
    var $_user;

    /**
     * Cached article objects.
     *
     * @var array $_cache
     */
    var $_cache = array();

    /**
     * Constructor.
     *
     * @access public
     */
    function Troll_Driver($user, $params = array())
    {
        $this->_user = $user;
        $this->_params = $params;
    }

    /**
     * Retrieve the specified data.
     *
     * @access public
     *
     * @param integer $group  The newsgroup to fetch articles for
     *
     * @return mixed  The specified object.
     */
    function retrieve($group)
    {
        return array();
    }

    /**
     * Store the specified data.
     *
     * @access public
     *
     * @param integer $group             The newsgroup to store articles for.
     * @param optional string $articles  Articles to store.
     *
     * @return mixed  The specified object.
     */
    function store($group, $articles = null)
    {
        return true;
    }

    /**
     * Attempts to return a concrete Troll_Driver instance based on $driver.
     *
     * @param string $driver          The type of concrete Troll_Driver
     *                                subclass to return. This is based on
     *                                the storage driver ($driver). The code
     *                                is dynamically included.
     * @param string $user            The name of the user who owns these
     *                                articles.
     * @param optional array $params  A hash containing any additional
     *                                configuration or connection parameters
     *                                a subclass might need.
     *
     * @return mixed  The newly created concrete Troll_Driver instance, or
     *                false on an error.
     */
    function &factory($driver, $user, $params = array())
    {
        $driver = basename($driver);
        if (empty($driver)) {
            return $driver = &new Troll_Driver($user, $params);
        }

        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Troll_Driver_' . $driver;
        if (class_exists($class)) {
            return $troll = &new $class($user, $params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Troll_Driver
     * instance based on $driver. It wil only create a new instance if
     * no Troll_Driver instance with the same parameters currently
     * exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Troll_Driver::singleton();
     *
     * @param string $driver          The type of concrete Troll_Driver
     *                                subclass to return. The is based on
     *                                the storage driver ($driver). The code
     *                                is dynamically included.
     * @param string $user            The name of the user who owns these
     *                                articles.
     * @param optional array $params  A hash containing any additional
     *                                configuration or connection parameters
     *                                a subclass might need.
     *
     * @return mixed    The created concrete Troll_Driver instance, or false
     *                  on error.
     */
    function &singleton($driver, $user, $params = array())
    {
        static $instances = array();

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Troll_Driver::factory($driver, $user, $params);
        }

        return $instances[$signature];
    }

}
