<?php

require_once 'Horde/Identity.php';

/**
 * This class provides a Troll-specific interface to all identities a
 * user might have. Its methods take care of any site-specific
 * restrictions configured in prefs.php and conf.php.
 *
 * $Horde: troll/lib/Identity/troll.php,v 1.4 2005/01/03 14:35:36 jan Exp $
 *
 * Copyright 2004-2005 Jan Schneider <jan@horde.org>
 * Copyright 2004-2005 Chuck Hagenbuch <chuck@horde.org>
 *
 * @author  Jan Schneider <jan@horde.org>
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.4 $
 * @since   Troll 0.1
 * @package Horde_Identity
 */
class Identity_troll extends Identity {

    /**
     * Returns an array with the necessary values for the identity select
     * box in the IMP compose window.
     *
     * @return array  The array with the necessary strings
     */
    function getSelectList()
    {
        $ids = $this->getAll('id');
        foreach ($ids as $key => $id) {
            $list[$key] = $this->getFromAddress($key) . ' (' . $id . ')';
        }
        return $list;
    }

    /**
     * Returns the from address based on the chosen identity. If no
     * address can be found it is built from the current user name and
     * the specified maildomain.
     *
     * @param optional integer $ident  The identity to retrieve the address
     *                                 from.
     *
     * @return string  A valid from address.
     */
    function getFromAddress($ident = null)
    {
        global $imp;

        if (!empty($this->_fromList[$ident])) {
            return $this->_fromList[$ident];
        }

        $val = $this->getValue('from_addr', $ident);
        if (empty($val)) {
            $val = $imp['user'];
        }

        if (!strstr($val, '@')) {
            $val .= '@' . $imp['maildomain'];
        }

        $this->_fromList[$ident] = $val;

        return $val;
    }
}
