<?php

require_once 'Horde/MIME/Contents.php';
require_once TROLL_BASE . '/config/mime_drivers.php';

/**
 * The Troll_Contents:: class extends the MIME_Contents:: class and contains
 * all functions related to handling the content and output of mail messages
 * in Troll.
 *
 * $Horde: troll/lib/MIME/Contents.php,v 1.36 2005/01/03 14:35:36 jan Exp $
 *
 * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
 * @version $Revision: 1.36 $
 * @since   Troll 0.0.4
 * @package Troll
 */
class Troll_Contents extends MIME_Contents {

    /**
     * The text of the body of the message.
     *
     * @var string $_body
     */
    var $_body = '';

    /**
     * The text of various MIME body parts.
     *
     * @var array $_bodypart
     */
    var $_bodypart = array();

    /**
     * The NNTP index of the message.
     *
     * @var integer $_index
     */
    var $_index;

    /**
     * Constructor
     *
     * @access public
     *
     * @param integer $index  The NNTP message index.
     */
    function Troll_Contents($index)
    {
        $this->_index = $index;

        /* Make sure we have an NNTP stream. */
        $troll_nntp = &Troll_NNTP::singleton();
        $troll_nntp->changeGroup(null);

        /* Get the MIME_Message object for the given index. */
        require_once 'Horde/MIME/Structure.php';
        $ob = &MIME_Structure::parse(@imap_fetchstructure($_SESSION['troll']['stream'], $index, FT_UID));

        parent::MIME_Contents($ob, array('download' => 'download_attach', 'view' => 'view_attach'));
    }

    /**
     * Gets the text of the body of the message.
     *
     * @access public
     *
     * @return string  The text of the body of the message.
     */
    function getBody()
    {
        if (empty($this->_body)) {
            $this->_body = @imap_body($_SESSION['troll']['stream'], $this->_index, FT_UID);
        }

        return $this->_body;
    }

    /**
     * Gets the raw text for one section of the message.
     *
     * @access public
     *
     * @param integer $id  The ID of the MIME_Part.
     *
     * @return string  The text of the part.
     */
    function getBodyPart($id)
    {
        if (!isset($this->_bodypart[$id])) {
            $this->_bodypart[$id] = @imap_fetchbody($_SESSION['troll']['stream'], $this->_index, $id, FT_UID);
        }

        return $this->_bodypart[$id];
    }

    /**
     * Return the URL to the view.php page.
     *
     * @access public
     *
     * @param object MIME_Part &$mime_part  See MIME_Contents::urlView().
     * @param integer $actionID             See MIME_Contents::urlView().
     * @param optional array $params        See MIME_Contents::urlView().
     * The following parameter names will be overwritten by this function:
     *   index
     *
     * @return string  See MIME_Contents::urlView().
     */
    function urlView(&$mime_part, $actionID, $params = array())
    {
        global $group;

        /* Add the necessary local parameters. */
        $params['index'] = $this->_index;

        /* Should this be a download link? */
        $dload = (($actionID === 'download_attach') ||
                  ($actionID === 'download_render') ||
                  ($actionID === 'save_message'));

        return parent::urlView($mime_part, $actionID, $params, $dload);
    }

    /*
     * Generate a link to the view.php page.
     *
     * @access public
     *
     * @param object MIME_Part &$mime_part  See MIME_Contents::linkView().
     * @param integer $actionID             See MIME_Contents::linkView().
     * @param string $text                  See MIME_Contents::linkView().
     * @param optional array $params        See MIME_Contents::linkView().
     *
     * @return string  See MIME_Contents::linkView().
     */
    function linkView(&$mime_part, $actionID, $text, $params = array())
    {
        /* If this is a 'download_attach' or 'download_render' link,
           we do not want to show in a new window. */
        $dload = (($actionID == 'download_attach') ||
                  ($actionID == 'download_render'));

        /* Add additional parameters for 'download_attach'. */
        if ($actionID == 'download_attach') {
        }

        return parent::linkView($mime_part, $actionID, $text, $params, $dload);
    }

    /**
     * Rebuild the MIME_Part structure of a message from NNTP data.
     *
     * @access public
     *
     * @param optional boolean $decode  Decode the body parts?
     *
     * @return object MIME_Message  A MIME_Message object with all of the body
     *                              text stored in the individual MIME_Parts.
     */
    function rebuildMessage($decode = false)
    {
        $part = $this->_message->getBasePart();
        $this->_rebuildMessage($part, $decode);

        return $this->_message;
    }

    /**
     * Recursive function used to rebuild the MIME_Part structure of a
     * message.
     *
     * @access private
     *
     * @param object MIME_Part &$part  A MIME_Part object.
     * @param boolean $decode          Decode the body parts?
     */
    function _rebuildMessage(&$part, $decode)
    {
        $id = $part->getMIMEId();

        if ($part->getPrimaryType() == 'multipart') {
            /* Recursively process any subparts. */
            foreach ($part->getParts() as $mime) {
                $this->_rebuildMessage($mime, $decode);
            }
        } elseif ($part->getType() == 'message/rfc822') {
            $part->setContents($this->getBodyPart(substr($id, 0, strrpos($id, '.'))));
            $this->_message->alterPart($id, $part);
        } else {
            if ($decode) {
                $this->_message->alterPart($id, $this->getDecodedMIMEPart($id));
            } else {
                $mime_part = &$this->getMIMEPart($id);
                $mime_part->setContents($this->getBodyPart($id));
                $this->_message->alterPart($id, $mime_part);
            }
        }
    }

    /**
     * Render a MIME Part.
     *
     * @param object MIME_Part &$mime_part  A MIME_Part object.
     * @param optional boolean $usecache    Use a cached MIME_Viewer object?
     *
     * @return string  The rendered data.
     */
    function renderMIMEPart(&$mime_part, $usecache = true)
    {
        if (!$mime_part->getContents()) {
            $mime_part->setContents($this->getBodyPart($mime_part->getMIMEId()));
        }
        return parent::renderMIMEPart($mime_part, $usecache);
    }

}
