<?php
/**
 * The Troll_MIME_Viewer_plain class renders out text/plain MIME parts with
 * URLs made into hyperlinks.
 *
 * $Horde: troll/lib/MIME/Viewer/plain.php,v 1.22 2005/01/03 14:35:37 jan Exp $
 *
 * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Anil Madhavapeddy <anil@recoil.org>
 * @version $Revision: 1.22 $
 * @since   Troll 0.0.4
 * @package Horde_MIME_Viewer
 */
class Troll_MIME_Viewer_plain extends MIME_Viewer {

    /**
     * Render out the currently set contents.
     *
     * @access public
     *
     * @param array $params  An array with a reference to a MIME_Contents
     *                       object.
     *
     * @return string  The rendered text in HTML.
     */
    function render($params)
    {
        $contents = &$params[0];

        global $prefs;

        $text = $this->mime_part->getContents();

        /* If calling as an attachment from view.php, we do not want to alter
           the text in any way. */
        if ($contents->viewAsAttachment()) {
            return $text;
        }

        if ($text === false) {
            return $contents->formatStatusMsg(_("There was an error displaying this message part"));
        }

        $text = rtrim($text);

        if ($text == '') {
            return $contents->formatStatusMsg(_("There was no text in this message part"));
        }

        /* Apply text filters. */
        require_once 'Horde/Text/Filter.php';

        $filters = array('text2html');
        $filter_params = array(array('parselevel' => TEXT_HTML_MICRO,
                                     'charset' => $this->mime_part->getCharset()));

        /* Highlight quoted parts of an email. */
        if ($prefs->getValue('highlight_text')) {
            $filters[] = 'highlightquotes';
            $filter_params[] = array();
        }

        /* Dim signatures. */
        if ($prefs->getValue('dim_signature')) {
            $filters[] = 'dimsignature';
            $filter_params[] = array();
        }

        $text = Text_Filter::filter($text, $filters, $filter_params);

        /* Wordwrap. */
        $text = str_replace("\t", '        ', $text);
        $text = str_replace('  ', ' &nbsp;', $text);
        $text = str_replace("\n ", "\n&nbsp;", $text);
        if ($text[0] == ' ') {
            $text = '&nbsp;' . substr($text, 1);
        }
        $text = '<p class="fixed">' . $text . '</p>';

        return $text;
    }

}
