<?php
/**
 * Troll Base Class
 *
 * $Horde: troll/lib/Troll.php,v 1.93 2005/01/03 14:35:35 jan Exp $
 *
 * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 2002-2005 Jon Parise <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.

 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@horde.org>
 * @version $Revision: 1.93 $
 * @package Troll
 */
class Troll {

    /**
     * Determine the backend to use.
     *
     * This decision is based on the global 'SERVER_NAME' and 'HTTP_HOST'
     * server variables and the contents of the 'preferred' either field
     * in the backend's definition.  The 'preferred' field may take a
     * single value or an array of multiple values.
     *
     * @access public
     *
     * @return array  The backend entry. Calls Horde::fatal() on error.
     */
    function getBackend()
    {
        include TROLL_BASE . '/config/backends.php';

        if (!isset($backends) || !is_array($backends)) {
            Horde::fatal(PEAR::raiseError(_("No backends configured in backends.php")), __FILE__, __LINE__);
        }

        foreach ($backends as $temp) {
	    if (!isset($backend)) {
                $backend = $temp;
            } elseif (!empty($temp['preferred'])) {
                if (is_array($temp['preferred'])) {
                    foreach ($temp['preferred'] as $val) {
                        if (($val == $_SERVER['SERVER_NAME']) ||
                            ($val == $_SERVER['HTTP_HOST'])) {
                            $backend = $temp;
                        }
                    }
                } elseif (($temp['preferred'] == $_SERVER['SERVER_NAME']) ||
                          ($temp['preferred'] == $_SERVER['HTTP_HOST'])) {
                    $backend = $temp;
                }
            }
        }

        /* Check for valid backend configuration. */
        if (!isset($backend)) {
            Horde::fatal(PEAR::raiseError(_("No backend configured for this host")), __FILE__, __LINE__);
        } elseif (empty($backend['driver'])) {
            Horde::fatal(PEAR::raiseError(sprintf(_("No '%s' element found in backend configuration."), 'driver')), __FILE__, __LINE__);
        }

        /* Make sure the 'params' entry exists. */
        if (!isset($backend['params'])) {
            $backend['params'] = array();
        }

        return $backend;
    }

    /**
     * Return the current news server string.
     *
     * @access public
     *
     * @return string  Newsserver string.
     */
    function serverString()
    {
        return '{' . $GLOBALS['conf']['news']['server'] . '/nntp}';
    }

    /**
     * Filter a string, if requested in preferences.
     *
     * @access public
     *
     * @param string $text  The text to filter.
     *
     * @return string  The filtered text (if requested).
     */
    function filterText($text)
    {
        global $conf, $prefs;

        if ($prefs->getValue('filtering')) {
            require_once 'Horde/Text/Filter.php';
            $text = Text_Filter::filter($text, 'words', array('words_file' => $conf['msg']['filtering']['words'], 'replacement' => $conf['msg']['filtering']['replacement']));
        }

        return $text;
    }

    /**
     * Build Troll's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $registry, $group, $prefs, $browser;

        require_once 'Horde/Menu.php';

        $target = (strpos($_SERVER['PHP_SELF'], 'groupview.php') !== false) ? '_parent' : '';
        $menu = &new Menu();

        /* Troll menu items. */
        $menu->add(Horde::applicationUrl('newsgroups.php'), _("Browse"), 'newsgroups/newsgroup.png', null, $target, null, basename($_SERVER['PHP_SELF']) == 'index.php' ? 'current' : null);
        $menu->add(Util::addParameter(Horde::applicationUrl('article_frames.php'), 'group', $prefs->getValue('last_group')), _("Last Group"), 'newsgroups/newsgroup.png', null, $target);

        $post_url = Horde::applicationUrl('post.php');
        if (isset($group)) {
            $post_url = Util::addParameter($post_url, 'group', $group);
        }

        Horde::addScriptFile('open_post_win.js');
        $menu->add('javascript:open_post_win(\'' . $browser->escapeJSCode(substr($post_url, 1)) . '\');', _("Post Message"), 'post.png', null, '_troll_compose');

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}

function GetHeaderIndex($message_id = 0, $indentlevel = 0, $visible = true, $subject = "None", $author = "Anonymous", $date = "none", $size = 0, $newa = true, $unreada = true, $hasreplies = false, $collapsed = false, $articlenumber = 0, $replies = 0, $url = "", $add = "", $plw = "100%")
{
    global $nslines, $nodhtml, $maxresults, $curresults, $group, $articles, $art_anchor,
        $firstprinted, $lastprinted, $thread, $totalexpand, $edt, $browser, $registry;

    if (strlen($subject) <= 1 ) {
        return;
    }

    $rtn = array();
    $rtn['nodhtml'] = $nodhtml;
    $rtn['message_id'] = $message_id;
    $rtn['item_id'] = "item_" . $message_id;
    $rtn['author'] = $author;
    $rtn['replies'] = $replies;
    $rtn['arrange'] = 0;
    $rtn['onclick'] = '';
    $rtn['indentlevel'] = $indentlevel;
    $rtn['plw'] = $plw;

    if (strlen($url) > 0) {
        $rtn['target'] = "";
        $rtn['article_url'] = $url;
    } else {
        $rtn['target'] = 'target="article"';
        $article_url = Horde::applicationUrl('article.php');
        $article_url = Util::addParameter($article_url, 'group=' . htmlspecialchars($group));
        $article_url = Util::addParameter($article_url, 'article=' . $articlenumber);
        $rtn['article_url'] = Util::addParameter($article_url, 'thread=' . $thread);
    }

    if ($visible) {
        $curresults++;
    }

    if ($articlenumber > $lastprinted) {
        $lastprinted = $articlenumber;
    }

    if ($articlenumber < IntVal($firstprinted)) {
        $firstprinted = $articlenumber;
    }

    $articles[]   = $articlenumber;
    $art_anchor[] = $message_id;

    $rtn['date'] = strftime('%x %X', strtotime($date));

    if ($rtn['indentlevel'] == -1) {
        $rtn['indentlevel'] = strlen($message_id) / 2 - 1;
    }

    if (!$nodhtml) {
        if ($browser->isBrowser('msie') || $browser->isBrowser('mozilla')) {
            $rtn['style'] = "display: " . ($visible ? 'block' : 'none');
        } else {
            $rtn['style'] = "position: absolute; visibility: " . ($visible ? 'show' : 'hide') . ";";
        }
    }

    $rtn['align'] = ($browser->isBrowser('msie')) ? 'middle' : 'baseline';

    if ($hasreplies && $nodhtml) {
        $rtn['collapsimg'] = $collapsed ? 'plus.png' : 'minus.png';
    } else {
        $rtn['collapsimg'] = 'blank.png';
    }

    if (!$nodhtml) {
        $rtn['onclick'] = "onclick=\"MarkRead('$message_id');\"";
    }

    $rtn['img'] = $unreada ? "graphics/message_closed.png" : "graphics/message_open.png";

    $rtn['subject'] = $subject . $add;

    if ($newa) {
        $rtn['subject'] = "<b>" . $subject . "</b>";
    }

    if(($rtn['indentlevel'] == 0 && $thread == 1 && $replies > 0)) {
        $overview_url = Horde::applicationUrl('threadoverview.php');
        $overview_url = Util::addParameter($overview_url, 'group=' . htmlspecialchars($group));
        $overview_url = Util::addParameter($overview_url,
                                          'article=' . $articlenumber);
        $rtn['overview_url'] = Util::addParameter($overview_url, 'thread=' . $thread);
    }

    if ($browser->isBrowser('mozilla') && $visible && !$nodhtml) {
        $rtn['arrange'] = 1;
    }

    return $rtn;
}

// --------------------------------------------------------------------
    function totalkidcount($a)
    {
        global $article_kids;

        if (!isset($article_kids[$a])) {
            return 0;
        }

        $t = 0;
        for ($i = 0; $i < count($article_kids[$a]); $i++) {
            $t = $t + totalkidcount($article_kids[$a][$i]);
        }

        return $t + count($article_kids[$a]);
    }

// ---------------------------------------------------------------------
function CreateId($a)
{
    global  $indentlevel, $article_parent, $article_kids;

    if (empty($indentlevel[$a])) {
        return "a$a";
    } else {
        $parentid = $article_parent[$a];
        for ($i=0; $i < count($article_kids[$parentid]); $i++) {
            if ($article_kids[$parentid][$i] == $a) {
                $i++;
                return  CreateId($parentid) . "a$i";
            }
        }
    }

    return '??';
}

function GetArticleTree($artuid)
{
    global $indentlevel, $overview, $article_kids, $showtree, $maxresults, $curresults, $totalexpand, $last_read, $numtokey;

    if (!isset($numtokey[$artuid])) {
        return array();
    }
    $key = $numtokey[$artuid];
    if (empty($overview[$key])) {
        return array();
    }

    $art_header = $overview[$key];
    $artuid = $art_header->uid;

    $id = CreateId($artuid);

    $visible = empty($indentlevel[$artuid]) ? true : false;
    if ($showtree == 1) {
        $visible = true;
    }

    $subject    = $art_header->subject;
    $author     = $art_header->from;
    $size       = $art_header->size;
    $date       = $art_header->date;

    $replies = totalkidcount($artuid);
    $hasreplies = ($replies == 0) ? false : true;
    if ($showtree == 1 || (isset($totalexpand[$artuid]) && $totalexpand[$artuid] == 1)) {
        $collapsed = false;
    } else {
        $collapsed = true;
    }

    if (!empty($totalexpand[$artuid])) {
        $visible = true;
    }

    $art_status = GetArticleStatus($key, $artuid, $last_read);
    $anew = $art_status['Recent'];
    $aunread = $art_status['Unseen'];

    if (($curresults > $maxresults) && IntVal($indentlevel[$artuid]) == 0) {
        return array();
    }

    $rtn = array();
    array_push($rtn, GetHeaderIndex($id, $indentlevel[$artuid], $visible, $subject, $author,
                                    $date, $size, $anew, $aunread, $hasreplies, $collapsed,
                                    $artuid, $replies));

    for ($i = 0; $i < @count($article_kids[$artuid]); $i++) {
        if (strlen($article_kids[$artuid][$i]) > 0) {
            array_push($rtn, GetArticleTree($article_kids[$artuid][$i]));
        }
    }

    return $rtn;
}

function GetArticle($stream, $number)
{
    global $group, $edt, $browser, $thread, $prefs, $articles, $art_anchor, $mime_drivers, $mime_drivers_map;

    require_once TROLL_BASE . '/lib/MIME/Contents.php';

    $rtn = array();

    $headers = @imap_headerinfo($stream, imap_msgno($stream, $number));
    if (!$headers) {
        Horde::fatal(PEAR::raiseError("Error on imap_headerinfo: " . imap_last_error()), __FILE__, __LINE__, true);
    }

    // Mark the post as read.
    MarkRead($number . ':' . $number);

    $author            = $headers->fromaddress;
    $rtn['messageid']  = $headers->message_id;
    $rtn['date']       = $headers->date;
    $rtn['from']       = $author;
    $rtn['subject']    = $headers->subject;
    $rtn['newsgroups'] = $headers->newsgroups;

    if (isset($headers->followup_to)) {
        $rtn['followup2']  = $headers->followup_to;
    }
    if (isset($headers->references)) {
        $rtn['references'] = $headers->references;
        $temp = explode(" ", $rtn['references']);
        $rtn['firstreference'] = rawurlencode($temp[0]);
    } else {
        $rtn['firstreference'] = '';
    }

    $troll_contents = &new Troll_Contents($number);
    if (!$troll_contents->buildMessage()) {
        Horde::fatal(PEAR::raiseError("Error on buildMessage()"), __FILE__, __LINE__, true);
    }
    $rtn['body'] = $troll_contents->getMessage();
    $rtn['rawbody'] = $troll_contents->getBody();

    $rtn['author'] = Horde::link($GLOBALS['registry']->call('mail/compose', array(array('to' => $author))), $author) . htmlspecialchars($author) . '</a>';

    if (is_array($articles)){
        $rtn['ind_art'] = array_search($number, $articles);
    } else {
        $rtn['ind_art'] = null;
    }

    $rtn['post_url'] = Horde::applicationUrl('post.php');
    $rtn['post_url'] = Util::addParameter($rtn['post_url'], array('article' => $number,
                                                                  'group' => htmlspecialchars($group)));

    return $rtn;
}

function GetThreadTree($article_number)
{
    global $indentlevel, $overview, $article_kids, $showtree, $maxresults, $curresults, $totalexpand, $last_read;

    $rtn = array();
    array_push($rtn, GetArticle($_SESSION['troll']['stream'], $article_number, $indentlevel[$article_number], "100%"));

    if (isset($article_kids[$article_number])) {
        for ($i = 0; $i < count($article_kids[$article_number]); $i++) {
            if (strlen($article_kids[$article_number][$i]) > 0) {
                array_push($rtn, GetThreadTree($article_kids[$article_number][$i]));
            }
        }
    }

    return $rtn;
}

function GetThreadOverview($artuid)
{
    global $indentlevel, $overview, $article_kids, $showtree, $maxresults, $curresults, $totalexpand, $last_read, $nodhtml, $fontsize, $entry, $numtokey;

    $visible = $indentlevel[$artuid] == 0 ? true : false;
    if ($showtree == 1) {
        $visible = true;
    }

    $key     = $numtokey[$artuid];
    $headers = $overview[$key];

    $subject = $headers->subject;
    $author  = $headers->from;
    $size    = $headers->size;
    $date    = $headers->date;

    $hasreplies = totalkidcount($artuid) == 0 ? false : true;
    $collapsed = false;
    $visible = true;
    $replies = 0;

    $art_status = GetArticleStatus($key, $artuid, $last_read);
    $anew    = $art_status['Recent'];
    $aunread = $art_status['Unseen'];

    $oldnodhtml = $nodhtml;
    $nodhtml  = 1;
    $fontsize = 2;

    if ($entry == $artuid) {
        $add = "<font color=red>&nbsp;<b>&lt; " . _("entry") . "</b></font>";
    } else {
        $add = "";
    }

    $rtn = array();
    array_push($rtn, GetHeaderIndex($artuid, $indentlevel[$artuid], $visible, $subject, $author,
                                    $date, $size, $anew, $aunread, $hasreplies, false, $artuid,
                                    $replies, "#a$artuid", $add, "100%"));

    $hdr = $rtn[0];
    // print out header
    require TROLL_TEMPLATES . '/threadoverview/article-index.inc';


    $nodhtml = $oldnodhtml;

    if (isset($article_kids[$artuid])) {
        for ($i = 0; $i < count($article_kids[$artuid]); $i++) {
            if (strlen($article_kids[$artuid][$i]) > 0) {
                array_push($rtn, GetThreadOverview($article_kids[$artuid][$i]));
            }
        }
    }

    return $rtn;
}

function FlatThreadOverview($artuid)
{
    global $overview, $fontsize, $nodhtml, $last_read, $id, $indentlevel, $keytonum;

    $rtn = array();

    $total = count($overview);
    end($overview);
    for ($i = 0; $i < $total; $i++) {
        $key = key($overview);
        if (($key !== 'count') && isset($keytonum[$key])) {
            $headers = current($overview);
            $subject = $headers->subject;
            $author  = $headers->from;
            $size    = $headers->size;
            $date    = $headers->date;
            $uid     = $headers->uid;

            $collapsed = false;
            $visible   = true;
            $replies   = 0;

            $art_status = GetArticleStatus($key, $artuid, $last_read);
            $anew = $art_status['Recent'];
            $aunread = $art_status['Unseen'];

            if ($artuid == $uid) {
                $add = "<font color=red>&nbsp;<b>&lt; " . _("entry") . "</b></font>";
            } else {
                $add = '';
            }

            $oldnodhtml = $nodhtml;
            $nodhtml = 1;
            $fontsize = 2;
            array_push($rtn, GetHeaderIndex($id, $indentlevel[$artuid], $visible, $subject, $author,
                                            $date, $size, $anew, $aunread, false, false, $artuid, $replies,
                                            '#a' . $key, $add, '100%'));
            $nodhtml = $oldnodhtml;
        }

        prev($overview);
    }

    return $rtn;
}

function FlatThreadTree()
{
    global $group, $overview, $keytonum;

    $total = count($overview);
    end($overview);

    $rtn = array();
    for ($i = 0; $i < $total; $i++) {
        $key = key($overview);
        if (($key !== 'count') && (isset($keytonum[$key]))) {
            $headers = current($overview);
            $subject = $headers->subject;
            $author  = $headers->from;
            $size    = $headers->size;
            $date    = $headers->date;

            $article_number = $keytonum[$key];
            array_push($rtn, GetArticle($_SESSION['troll']['stream'], $article_number, 0, '100%'));

        }

        prev($overview);
    }

    return $rtn;
}

/**
 * GetArticleStatus: return an array of boolean ['Recent', 'Unseen']
 */
function GetArticleStatus($key, $artnum, $last_read)
{
    global $conf, $overview;
    $status = array();

    if ($conf['news']['subscribe']) {
        if (isset($overview)) {
            $art_header = $overview[$key];
            $status['Recent'] = ($art_header->recent != ' ')  ? true : false;
            $status['Unseen'] = ($art_header->deleted == ' ') ? true : false;
        } else {
            Horde::logMessage("GetArticleStatus: overview not set.", __FILE__, __LINE__, PEAR_LOG_DEBUG);
            $status['Recent'] = true;
            $status['Unseen'] = true;
        }
    } else {
        if ($artnum > $last_read) {
            $status['Recent'] = true;
            $status['Unseen'] = true;
        } else {
            $status['Recent'] = false;
            $status['Unseen'] = false;
        }
    }

    return $status;
}

    /**
     * Prune article list to not show read messages
     *
     * @access public
     *
     * @param array $list the current article list
     * @param string $group the newsgroup
     *
     * @return array new list of articles
     */
    function RemoveReadArticles($list, $group)
    {
        global $conf, $troll_driver;

        if (!$conf['news']['subscribe']) {
            // just return everything
            return $list;
        }

        // get current read article list
        $current_messages = $troll_driver->retrieve($group);
        if (array_key_exists($group, $current_messages)) {
            // turn article array into a string since the array
            // can be large enough to cause php to
            // run out of memory
            $art = arrayToRange($list);
            // find the messages in $art that are no in $current
            $rst = rangeUnion($art, explode(",", $current_messages[$group]));
            // turn the resulting range back into an array
            $list = RangeToArray($rst);
        }

        return $list;
    }

    /**
     * Convert a message range into an array containing everything
     * within that range
     *
     * @access public
     *
     * @param array $arr an array of message numbers and/or ranges
     *
     * @return array new list of articles
     */
    function RangeToArray($arr)
    {
        $r = array();

        foreach ($arr as $a) {
            if (strpos($a, ":") !== false) {
                $t = explode(":", $a);
                for ($i = $t[0]; $i <= $t[1]; $i++) {
                    array_push($r, $i);
                }
            } else {
                array_push($r, $a);
            }
        }

        return $r;
    }

    /**
     * Take a message list and remove messages that have already been
     * read
     *
     * @access public
     *
     * @param array $in the complete message list
     * @param array $out messages to remove from $in
     *
     * @return array new list of articles
     */
    function rangeUnion($in, $out)
    {
        foreach ($in as $k => $i) {
            if (strpos($i, ":") !== false) {
                $r = explode(":", $i);
                $min = $r[0];
                $max = $r[1];
            } else {
                $min = $i;
                $max = $i;
            }

            foreach ($out as $o) {
                if (strpos($o, ":") !== false) {
                    $r = explode(":", $o);
                    if ($min >= $r[0]) {
                        if ($max <= $r[1]) {
                            unset($in[$k]);
                        } else {
                            $in[$k] = ($r[1] + 1) . ":" . $max;
                        }
                    } else {
                        // min < r[0]
                        if ($max >= $r[0]) {
                            if ($max == $min) {
                                unset($in[$k]);
                            } else {
                                $in[$k] = $min . ":" . ($r[0] - 1);
                            }
                        }
                    }
                } else {
                    if (($min >= $o) && ($max <= $o)){
                        unset($in[$k]);
                    }
                }
            }
        }

        unset($in[0]);
        return $in;
    }

    /**
     * Take an array and compact inclusive series into ranges
     * of the form min:max
     *
     * @access public
     *
     * @param array $arr the array to compact
     *
     * @return array new list of articles
     */
    function arrayToRange($arr)
    {
        $rtn = array();
        $last = $amin = $amax = next($arr);

        if ($last !== false) {
            foreach ($arr as $a) {
                if ($a == ($last + 1)) {
                    $amax = $a;
                } else {
                    if ($amax == $amin) {
                        array_push($rtn, $last);
                    } else {
                        array_push($rtn, $amin . ":" . $amax);
                    }
                    $amin = $amax = $a;
                }
                $last = $a;
            }
            if ($amax == $amin) {
                array_push($rtn, $last);
            } else {
                array_push($rtn, $amin . ":" . $amax);
            }
        }

        unset($rtn[0]);
        return $rtn;
    }

    /**
     * take a string if it is a range return an array with max/min
     * if it is not a range, return $str as max and min
     *
     * @access public
     *
     * @param string $str the string to explode
     *
     * @return array max in min from $str
     */
    function rangeToMaxMin($str)
    {
        if ($str === false){
            return false;
        }

        if (strpos($str, ":") == false){
            $r = array($str, $str);
        } else {
            $r = explode(":", $str);
        }

        return $r;
    }

    /**
     * take a range and merge it into an existing list of messages/ranges
     *
     * @access public
     *
     * @param string $val the range to merge
     * @param string $list a , separated list of messages/ranges
     *
     * @return string a comma separated list of messages/ranges
     */
    function MergeRanges($val, $list)
    {
        $m = explode(":", $val);
        $min = $m[0];
        $max = $m[1];

        // if there is no current list, just return the new value
        if (is_null($list)){
            return($val);
        }

        if (strpos($list, ",") !== false) {
            $array = explode(",", $list);
        } else {
            $array = array($list);
        }

        // prime min value
        $amin = rangeToMaxMin($array[0]);

        if (($min == ($amin[0] - 1)) || ($min == $amin[0])) {
            // replace first entry
            $max = ($amin[1] < $max) ? $max : $amin[1];
            $array[0] = $min . ":" . $max;

            // go through and remove anything b/w amin[1] and max
            // skipping first element
            foreach ($array as $k => $v) {
                $n = rangeToMaxMin($v);
                if ($n[1] < $max) {
                    unset($array[$k]);
                } else {
                    break;
                }
            }
            return implode(",", $array);
        } else if ($max < $amin[0]) {
            if ($min == $max) {
                array_unshift($array, $min);
            } else {
                array_unshift($array, $min . ":" . $max);
            }
            return implode(",", $array);
        } else if (($max <= $amin[1]) && ($min >= $amin[0])) {
            // do nothing
            return implode(",", $array);
        }

        $amax = rangeToMaxMin(next($array));

        $done = false;
        // list is already sorted
        while (list($k, $v) = each($array)) {
            $amin = $amax;
            $amax = rangeToMaxMin($v);

            if ($min == ($amin[1] + 1)) {
                // merge several records together
                if ($max == ($amax[0] - 1)) {
                    // merge all 3
                    $array[$k - 1] = $amin[0] . ":" . $amax[1];
                    unset($array[$k]);
                } else {
                    // merge min
                    $array[$k - 1] = $amin[0] . ":" . $max;
                }
                $done = true;
                break;
            } else if ($max == ($amax[0] - 1)) {
                // merge max
                $array[$k] = $min . ":" . $amax[1];
                $done = true;
                break;
            } else if (($min > $amin[0]) && ($amax[0] > $max)) {
                // insert into list
                $tmp1 = (($min == $max) ? $min : ($min . ":" . $max));
                for($i = $k; $i < count($array); $i++) {
                    $tmp2 = $array[$i];
                    $array[$i] = $tmp1;
                    $tmp1 = $tmp2;
                }
                $done = true;
                break;
            }
        }

        if (!$done) {
            // push onto end
            if ($max == $min) {
                array_push($array, $min);
            } else {
                array_push($array, $min . ":" . $max);
            }
        }

        return implode(",", $array);
    }

    /**
     * take a range of messages and mark them as read
     *
     * @access public
     *
     * @param string $messages a range of messages
     *
     * @return boolean or Pear error
     */
    function MarkRead($messages)
    {
        global $conf, $group, $troll_driver;

        if (!$conf['news']['subscribe']) {
            // don't save read message status
            return false;
        }

        // get current read article list
        $current_messages = $troll_driver->retrieve($group);
        if (array_key_exists($group, $current_messages)) {
            $new_list = MergeRanges($messages, $current_messages[$group]);
        } else {
            $new_list = $messages;
        }

        $rtn = true;
        $rtn = $troll_driver->store($group, $new_list);

        return $rtn;
    }

/**
 * Mark all messages as read (i.e. "deleted") until the given argument.
 */
function MarkReadUntil($maxnum)
{
    $rtn = MarkRead("1:" . $maxnum);
    return $rtn;
}

/**
 * Return the first uid of the current stream.
 */
function ImapFirst()
{
    global $articles;

    return isset($articles[0]) ? $articles[0] : 0;
}

/**
 * Return the last uid of the current stream.
 */
function ImapLast()
{
    global $articles;

    $maxind = count($articles) - 1;

    return isset($articles[$maxind]) ? $articles[$maxind] : 0;
}
