<?php
/**
 * $Horde: troll/newsgroups.php,v 1.76 2005/01/03 14:35:35 jan Exp $
 *
 * Copyright 2000-2005 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 2000-2005 Jon Parise <jon@horde.org>
 * Copyright 2000-2005 Anil Madhavapeddy <avsm@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/**
 * Utility function to return a url for the various newsgroup tree
 * images.
 *
 * @param string $name  Image filename.
 * @param string $alt   Image alt tag.
 * @param string $type  Either 'icon' or 'tree'.
 *
 * @return string  Link to the folder image.
 */
function _image($name, $alt, $type)
{
    static $cache = array();

    if (!empty($cache[$type][$name])) {
        return $cache[$type][$name];
    }

    if ($type == 'icon') {
        $cache[$type][$name] = Horde::img('newsgroups/' . $name, $alt, 'border="0" width="16" height="16" style="vertical-align:middle"');
    } else {
        $cache[$type][$name] = Horde::img('tree/' . $name, $alt, 'width="20" height="20" style="vertical-align:middle"', $GLOBALS['registry']->getImageDir('horde'));
    }

    return $cache[$type][$name];
}

@define('TROLL_BASE', dirname(__FILE__));
require_once TROLL_BASE . '/lib/base.php';
require_once TROLL_BASE . '/lib/NNTP.php';
require_once TROLL_BASE . '/lib/IMAP/Tree.php';
require_once 'Horde/Template.php';

/* Make sure we have an NNTP stream. */
$troll_nntp = &Troll_NNTP::singleton();
$troll_nntp->changeGroup(null);

/* Create the Troll_Tree object. */
$trolltree = &Troll_Tree::singleton(true);

$newsgroup_list = Util::getFormData('newsgroup_list');
$refresh_time = false;
$showAll = (!$conf['news']['subscribe'] || !empty($_SESSION['troll']['showall']));

/* Run through the action handlers. */
$actionID = Util::getFormData('actionID');
switch ($actionID) {
case 'collapse_newsgroup':
case 'expand_newsgroup':
    $newsgroup = Util::getGet('newsgroup');
    if (!empty($newsgroup)) {
        ($actionID == 'expand_newsgroup') ? $trolltree->expand($newsgroup) : $trolltree->collapse($newsgroup);
    }
    break;

case 'expand_all_newsgroups':
    $trolltree->expandAll();
    break;

case 'collapse_all_newsgroups':
    $trolltree->collapseAll();
    break;

case 'refresh_newsgroups':
    $trolltree->init(IMAPTREE_INIT_UNSUB);
    break;

case 'toggle_subscribed_view':
    if ($conf['news']['subscribe']) {
        $showAll = !$showAll;
        $_SESSION['troll']['showall'] = $showAll;
        $trolltree->showUnsubscribed($showAll);
    }
    break;

case 'subscribe_newsgroup':
    if (is_array($newsgroup_list)) {
        $trolltree->subscribeNewsgroups($newsgroup_list);
        $notification->push(_("Subscribed to groups."), 'horde.success');
    }
    break;

case 'unsubscribe_newsgroup':
    if (is_array($newsgroup_list)) {
        $trolltree->unsubscribeNewsgroups($newsgroup_list);
        $notification->push(_("Unsubscribed from groups."), 'horde.success');
    }
    break;

case 'poll_newsgroup':
case 'nopoll_newsgroup':
     if (is_array($newsgroup_list)) {
        foreach ($newsgroup_list as $newsgroup) {
            if ($actionID == 'poll_newsgroup') {
                $trolltree->addPollList($newsgroup);
            } else {
                $trolltree->removePollList($newsgroup);
            }
        }
    }
    break;
}

/* Display the correct message on the action bar. */
$subToggleText = ($showAll) ? _("Hide Unsubscribed") : _("Show All Groups");

/* Set the URL to refresh the page too in the META tag. */
$refresh_url = Horde::selfUrl(false, true, true);

$title = _("Newsgroup Browser");
require TROLL_TEMPLATES . '/common-header.inc';
require TROLL_TEMPLATES . '/menu.inc';

$article_url = Horde::url('article_frames.php');
$news_url = Horde::url('newsgroups.php');

$first_elt = true;
$rowct = 0;
$moreelt = $rows = array();

require TROLL_TEMPLATES . '/newsgroups/javascript.inc';
require TROLL_TEMPLATES . '/newsgroups/head.inc';

$i = 0;
require TROLL_TEMPLATES . '/newsgroups/actions.inc';

/* Start iterating through the list of groups, displaying them. */
$elt = $trolltree->reset();
do {
    $row = array();
    $row['msgs'] = $row['new'] = '&nbsp;';
    $row['check'] = false;

    /* Determine if we need to poll this group for new messages. */
    if (!$trolltree->isContainer($elt)) {
        /* We are dealing with a group here.
         * Determine if we need to poll this group for new messages. */
        if ($trolltree->isPolled($elt)) {
            /* If we need message information for this folder, update it
             * now. */
            $msgs_info = $trolltree->getElementInfo($elt['value']);
            if (!empty($msgs_info)) {
                /* Identify the number of messages in the group. */
                $row['msgs'] = $msgs_info['messages'];

                /* Highlight mailboxes with unread messages in bold. */
                $row['new'] = $msgs_info['unseen'];
                if (!empty($row['new'])) {
                    $elt['label'] = '<b>' . $elt['label'] . '</b>';
                }
            }
        }

        $row['name'] = Horde::link(Util::addParameter($article_url, 'group', $elt['value']), sprintf(_("View messages in %s"), $elt['value'])) . $elt['label'] . '</a>';

        $dir2 = _image('newsgroup.png', _("Newsgroup"), 'icon');
        $row['check'] = true;
    } else {
        $row['name'] = $elt['label'];
        if ($trolltree->isOpen($elt)) {
            $dir2 = _image('newscollection.png', _("Open Newsgroup"), 'icon');
        } else {
            $dir2 = _image('newscollection.png', _("Closed Newsgroup"), 'icon');
        }
    }

    $peek = $trolltree->peek();

    if ($trolltree->hasChildren($elt, true)) {
        $dir = Util::addParameter($news_url, 'newsgroup', $elt['value']);
        if ($trolltree->isOpen($elt)) {
            $dir = Util::addParameter($dir, 'actionID', 'collapse_newsgroup');
            if ($first_elt) {
                $minus_img = 'minustop.png';
            } else {
                $minus_img = ($peek) ? 'minus.png' : 'minusbottom.png';
            }
            $dir = '<a name="' . $elt['value'] . '"></a>' . Horde::link($dir, _("Collapse Newsgroup")) . _image($minus_img, _("Collapse Newsgroup"), 'tree');
        } else {
            $dir = Util::addParameter($dir, 'actionID', 'expand_newsgroup');
            $dir .= '#' . urlencode($elt['value']);
            if ($first_elt) {
                $plus_img = ($peek) ? 'plustop.png' : 'plusonly.png';
            } else {
                $plus_img = ($peek) ? 'plus.png' : 'plusbottom.png';
            }
            $dir = Horde::link($dir, _("Expand Newsgroup")) . _image($plus_img, _("Expand Newsgroup"), 'tree');
        }
        $dir .= '</a>' . $dir2;
    } else {
        $join_img = ($peek) ? 'join.png' : 'joinbottom.png';
        $dir = _image($join_img, '', 'tree') . $dir2;
    }

    /* Highlight line differently if folder/mailbox is
     * unsubscribed. */
    if ($showAll && !$trolltree->isSubscribed($elt)) {
        $row['navclass'] = 'newsgroupunsub';
    } else {
        $row['navclass'] = (++$rowct % 2) ? 'item0' : 'item1';
    }

    $row['elt_val'] = htmlspecialchars($elt['value']);

    $row['line'] = '';
    $moreelt[$elt['level']] = $peek;
    for ($i = 0; $i < $elt['level']; $i++) {
        $row['line'] .= _image(($moreelt[$i]) ? 'line.png' : 'blank.png', '', 'tree');
    }
    $row['line'] .= $dir;

    $rows[] = $row;
    $first_elt = false;

} while (($elt = $trolltree->next()));

/* Render the rows now. */
$template = &new Horde_Template();
$template->set('rows', $rows, true);
echo $template->fetch(TROLL_TEMPLATES . '/newsgroups/newsgroups.html');
if ($rowct > 10) {
    $i = 1;
    require TROLL_TEMPLATES . '/newsgroups/actions.inc';
}

require TROLL_TEMPLATES . '/newsgroups/foot.inc';
require $registry->get('templates', 'horde') . '/common-footer.inc';
