<?php
/**
 * $Horde: troll/threadoverview.php,v 1.30 2005/01/03 14:35:35 jan Exp $
 *
 * Copyright 2000-2005 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 2000-2005 Jon Parise <jon@horde.org>
 * Copyright 2000-2005 Anil Madhavapeddy <avsm@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/* See if this is an article array or an array of article arrays. */
function _printArticles($list, $group)
{
    foreach ($list as $a) {
        if (isset($a['author'])) {
            $art = $a;
            require TROLL_TEMPLATES . '/article/article-headers.inc';
            echo $art['body'];
            require TROLL_TEMPLATES . '/article/article-footers.inc';
        } else {
            _printArticles($a, $group);
        }
    }
}

/* See if this is a header array or an array of header arrays. */
function _printThreadOverview($headers)
{
    foreach ($headers as $hdr) {
        if (isset($hdr['author'])) {
            require TROLL_TEMPLATES . '/threadoverview/article-index.inc';
        } else {
            _printThreadOverview($hdr);
        }
    }
}

@define('TROLL_BASE', dirname(__FILE__));
require_once TROLL_BASE . '/lib/base.php';
require_once TROLL_BASE . '/lib/NNTP.php';

$title = _("Thread Overview");
require TROLL_TEMPLATES . '/common-header.inc';
require TROLL_TEMPLATES . '/menu.inc';

$article = Util::getFormData('article');
$group = Util::getFormData('group');

$thread = Util::getFormData('thread');
if (empty($thread)) {
    if (empty($tdc_groups)) {
        $tdc_groups = '';
    }
    parse_str($tdc_groups);
}

$max = $maxresults = $prefs->getValue('max_articles');
$curresults = 0;
$nodhtml = 1;
$entry = $article;

/* Make sure we have an NNTP stream. */
$troll_nntp = &Troll_NNTP::singleton();
$troll_nntp->changeGroup($group);

$articles = @imap_search($_SESSION['troll']['stream'], ' ALL ', SE_UID);
if (!$articles) {
    Horde::logMessage("Error on imap_search: " . imap_last_error(), __FILE__, __LINE__, PEAR_LOG_NOTICE);
    $articles = array();
} else {
    sort($articles);
}

$last_read = 0;
$max *= (!empty($showtree)) ? 2 : 6;
$last = !empty($end) ? $end : ImapLast();
if (!empty($only_new)) {
    $first = $last_read + 1;
} else {
    $first = ($last > $max) ? ($last - $max) : 0;
}
$end = $last;
if ($first < 1) {
    $first = 1;
}

$firstprinted = $last;
$lastprinted = $first;

$msgid = Util::getFormData('msgid');
if (empty($msgid)) {
    $headers = @imap_headerinfo($_SESSION['troll']['stream'], imap_msgno($_SESSION['troll']['stream'], $article));
    if (!$headers) {
        Horde::fatal(PEAR::raiseError("Error on imap_headerinfo: " . imap_last_error()), __FILE__, __LINE__, true);
    }
    $msgid = $headers->message_id;
}

$overview = @imap_fetch_overview($_SESSION['troll']['stream'], implode(',', $articles), FT_UID);
if (!$overview) {
    Horde::fatal(PEAR::raiseError("Error on imap_fetch_overview: " . imap_last_error()), __FILE__, __LINE__, true);
}

if (is_array($overview)) {
    foreach ($overview as $key => $value) {
        $messageid = $value->message_id;
        if (!isset($value->references)) {
            $value->references = '';
        }

        if (($messageid != $msgid) && strpos($value->references, $msgid) === false) {
            // Not in thread.
            continue;
        }

        $messagenum = $value->uid;

        $keytonum[$key]        = $messagenum;
        $numtokey[$messagenum] = $key;
        $idtonum [$messageid]  = $messagenum;
        $numtoid [$messagenum] = $messageid;
        $haskids [$messagenum] = 0;

        // Parent M-ID.
        $parent = chop(strrchr($value->references, '<'));

        if ((strlen($parent) > 0) && isset($idtonum[$parent]) && $thread) {
            $x = IntVal($idtonum[$parent]);
            if ($x == 0) {
                $indentlevel[$messagenum] = 0;
                $kids[$messagenum] = 0;
            } else {
                $article_parent[$messagenum] = $x;
                $article_kids[$x][] = $messagenum;
                $indentlevel[$messagenum] = $indentlevel[$x] + 1;
            }

            if ($messagenum > $last_read) {
                $root = $messagenum;
                while ($indentlevel[$root] != 0 && $root > 0) {
                    if ($root > 0) {
                        $totalexpand[$root] = 1;
                    }
                    $root = $article_parent[$root];
                }
                if ($root > 0) {
                    $totalexpand[$root] = 1;
                }
            }
        } else {
            $indentlevel[$messagenum] = 0;
            $kids[$messagenum] = 0;
        }
    }
}

$t_last = ImapLast();
$t_first = ImapFirst();
$t_new = $last_read;
if ($t_new < $t_first) {
    $t_new = $t_first;
}

$a_old = $t_new - $t_first;
$a_new = $t_last - $t_new;
if (!empty($thread)) {
    $yes = '(' . _("Yes") . ')';
    $no  = _("No");
} else {
    $yes = _("Yes");
    $no  = '(' . _("No") . ')';
}

$emsgid = rawurlencode($msgid);

$thread_url = Horde::applicationUrl('threadoverview.php');
$thread_url = Util::addParameter($thread_url, array('group' => $group, 'msgid' => $emsgid, 'article' => $article));
$return_url = Util::addParameter(Horde::applicationUrl('article_frames.php'), 'group', $group);

if (empty($thread)) {
    require TROLL_TEMPLATES . '/threadoverview/header.inc';
    require TROLL_TEMPLATES . '/threadoverview/navbar.inc';
    require TROLL_TEMPLATES . '/threadoverview/article-header.inc';
    $headers = FlatThreadOverview($article);
    _printThreadOverview($headers);
    require TROLL_TEMPLATES . '/threadoverview/article-footer.inc';
    require TROLL_TEMPLATES . '/threadoverview/footer.inc';
    $article_list = FlatThreadTree();

    _printArticles($article_list, $group);
} else {
    foreach ($overview as $i => $value) {
        if (isset($keytonum[$i]) && ($indentlevel[$keytonum[$i]] == 0)) {
            require TROLL_TEMPLATES . '/threadoverview/header.inc';
            require TROLL_TEMPLATES . '/threadoverview/navbar.inc';
            require TROLL_TEMPLATES . '/threadoverview/article-header.inc';
            $headers = GetThreadOverview($keytonum[$i]);
            _printThreadOverview($headers);
            require TROLL_TEMPLATES . '/threadoverview/article-footer.inc';
            require TROLL_TEMPLATES . '/threadoverview/footer.inc';
            $article_list = GetThreadTree($keytonum[$i]);

            _printArticles($article_list, $group);
        }
    }
}

require $registry->get('templates', 'horde') . '/common-footer.inc';
